#include "mmGlobal.h"
#include "mmData.h"


dual MM_WriteData (char *FileName, PLOT_DATA *Data)
{
  int   count, Index;
  int   MaxLen = 0;
  FILE *fp = NULL;

  PRO_START ("["__FUNCTION__"]...")

  if (FileName == NULL || Data == NULL)
    PRO_RETURN ("Nothing to do", TRUE)

  if ((fp = fopen(FileName, "w+")) == NULL)
    PRO_RETURN ("file-opening failed", FALSE)

  fprintf (fp, "METEX\n");
  fprintf (fp, "# Version:\t0.2\n");
  fprintf (fp, "# Date:   \t\n");
  fprintf (fp, "# Comment:\t\n");
  fprintf (fp, "#\n");
  fprintf (fp, "\n");
  fprintf (fp, "Lines: %d\n", Data->Number);
  for (Index = 0; Index < Data->Number; ++Index)
    MaxLen = MAX (MaxLen, Data->Line[Index]->len);
  fprintf (fp, "Time:  %d\n", MaxLen);
  fprintf (fp, "\n");
  fprintf (fp, "# [Plot-Data:]\n");
  for (Index = 0; Index < Data->Number; ++Index)
    fprintf (fp, "%d: %s\n", Index, Data->Line[Index]->Name);
  fprintf (fp, "Mode:  ");
  for (Index = 0; Index < Data->Number; ++Index)
    switch (Data->Line[Index]->Mode)
    {
      case BAR_GRAPH :
	fprintf (fp, "      BAR");
	break;
      case SEG_DISPL :
	fprintf (fp, "     SEGM");
	break;
      case LINE_PLOT : 	
      default:
	fprintf (fp, "     LINE");
    }
  fprintf (fp, "\n");

  fprintf (fp, "Color: ");
  for (Index = 0; Index < Data->Number; ++Index)
    fprintf (fp, "     %4d", Data->Line[Index]->Color);
  fprintf (fp, "\n");
  fprintf (fp, "Unit:  ");
  fprintf (fp, "\n");
  fprintf (fp, "\n");

  fprintf (fp, "# [Data:]\n");
  for (count = 0; count < MaxLen; ++count)
  {
    fprintf (fp, "%5d:  ", count);
    for (Index = 0; Index < Data->Number; ++Index)
      if (count < Data->Line[Index]->len)
	fprintf (fp, " %10.3e", Data->Line[Index]->Val[count]);
      else
	fprintf (fp, "    ---    ");
    fprintf (fp, "\n");
  }

  fclose (fp);  

  PRO_END
  return TRUE;
}


PLOT_DATA *MM_ReadData (char *FileName, PLOT_DATA *Data)
{
  int    count, Index;
  int    Time, Number;
  int    Color;
  int    ID[256];
  char   letter = ' ';
  char   String[256];
  float  Val;
  FILE  *fp = NULL;

  PRO_START ("["__FUNCTION__"]...\n")

  if (FileName == NULL)
    PRO_RETURN ("Nothing to do", NULL)

  if ((fp = fopen(FileName, "r+")) == NULL)
    PRO_RETURN ("file-opening failed", NULL)

  fscanf (fp, "%s", String);
  if (strcmp("METEX", String) != 0)
    PRO_RETURN ("invalid file-type", NULL)

  if (Data == NULL)
    Data = MM_InitData ();

  do
    letter = fgetc (fp);
  while (letter != 'L');

  do
    letter = fgetc (fp);
  while (letter != ':');
  fscanf (fp, "%d", &Number);

  do
    letter = fgetc (fp);
  while (letter != ':');
  fscanf (fp, "%d", &Time);

  PRO2 ("Lines: %d  Time: %d\n", Number, Time)

  for (Index = 0; Index < Number; ++Index)
  {
    fscanf (fp, "%100s\n", String);
    ID[Index] = MM_AddData (Data, String, 0, LINE_PLOT);
  }

  do
    letter = fgetc (fp);
  while (letter != ':');

  for (Index = 0; Index < Number; ++Index)
  {
    fscanf (fp, "%s", String);
    if (strncmp("BAR", String, 3) == 0)
      Data->Line[ID[Index]]->Mode = BAR_GRAPH;
    elsif (strncmp("SEGM", String, 4) == 0)
      Data->Line[ID[Index]]->Mode = SEG_DISPL;
    else
      Data->Line[ID[Index]]->Mode = LINE_PLOT;
  }

  do
    letter = fgetc (fp);
  while (letter != ':');

  for (Index = 0; Index < Number; ++Index)
  {
    fscanf (fp, "%d", &Color);
    Data->Line[ID[Index]]->Color = Color;
  }

  do
    letter = fgetc (fp);
  while (letter != ']');
  do 
    letter = fgetc (fp);
  while (letter != ':');

  count = 0;
  while (count < Time && ! feof(fp))
  {
    do
      letter = fgetc (fp);
    while (letter != ':');
    fgetc (fp);
    fgetc (fp);
 
    for (Index = 0; Index < Number; ++Index)
    {
      fscanf (fp, "%s", String);
      if (strcmp(String, "---") != 0)
      {
	Val = atof (String);
	MM_AddValue (Data, ID[Index], Val);
      }
    }
    count++;
  }
  fclose (fp);

  PRO_END
  return Data;
}

