#ifndef MMGLOBAL_H
#define MMGLOBAL_H

#include <libprg.h>

/* #define DELAY 1000 */
#define MAX_FAIL 10

#define MSEC(x) ((x)*1000)

enum UNITS
{
  BAD_UNIT = -1,
  DC,         /* mV, V, A, mA, A */               /* V */
  AC,         /* mV, V, A, mA, A ? */             /* A */
  FR,         /* KHz, MHz */                       /* F */
  LO,         /* - rdy, rdy,  */                   /* L */
  OH,         /* Ohm, KOhm, MOhm, 0.L MOhm */      /* O */
  DI,         /* mV */                             /* D */
  CA,         /* nF, 0.F */                        /* C */
  HF,         /* */                                /*   */
  TE          /* C, 0L C */                        /* T */
};


extern char UnitStr[][3];
#define NUM_UNITS 9

enum RANGES
{
  BAD_RANGE = -1,
  NO_RANGE,  
  MV,
  V,
  MUA,
  MA,
  A,
  KHZ,
  MHZ,
  LOW,
  HIGH,
  OHM,
  KOHM,
  MOHM,
  NF,
  F,
  C
};    

extern char RangeStr[][6];
#define NUM_RANGES 15


enum FUNCTIONS
{
  NONE = 0,
  ADD,
  DIFF,
  MAX,
  MIN
};

enum TRIGGERS
{
  IMMEDIATE = 0,
  RISING_EDGE,
  FALLING_EDGE,
  LIMIT,
  DELAY
};
  

typedef struct METEX_VALUE
{
  short Unit;
  float Value;
  int   Range;
  dual  OL;
} METEX_VALUE, *PMETEX_VALUE;


typedef struct 
{
  int    Device;
  char   DevName[16];
  int    Speed;
  dual   Sync;      /* FALSE: Asynchron -> invalid value: inc(time) + 1 */
  int    Trigger;
  double Value;
  int    Delay;
} METEX_PARAM;


typedef void (*TIMERFUNCTION)(void *Param, METEX_VALUE Value);

typedef struct
{
  int            fd;
  int            Delay;
  short          Time;
  XtIntervalId   ID;
  int            failed;
  TIMERFUNCTION  Function;
  void          *Param;
} TIMER_DATA;


extern char        Devices[][16];
extern METEX_PARAM Param;


/* mmSerial: */
int          MM_OpenPort          (char *DevName);
void         MM_ClosePort         (int fd);
void         MM_InitPort          (int fd);
void         MM_TestPort          (int fd);
dual         MM_SendData          (int fd, char *Data);
char        *MM_RecvData          (int fd, char *Result, int len);
char        *MM_GetValue          (int fd);
int          MM_HaveData          (int fd);
void         MM_CleanPort         (int fd);

/* mmMetex: */
void         MM_ReadValue         (TIMER_DATA *Data, 
				   TIMERFUNCTION Function, void *Param);

/* mmWin: */
void         MM_OpenPlotWindow    (char *Title, METEX_PARAM Data);
void         MM_OpenBarWindow     (char *Title, METEX_PARAM Data);
void         MM_OpenSegmWindow    (char *Title, METEX_PARAM Data);

#endif
