#include <ctype.h>

#include <libdlg.h>

#include "mmGlobal.h"

#define TIMER_STEP 50

char UnitStr[][3] = {"DC", "AC", "FR", "LO", "OH", 
		     "DI", "CA", "HF", "TE"};
char RangeStr[][6] = {"", 
		      "mV",  "V",
		      "A",  "mA", "A",
		      "KHz", "MHz",
		      "- rdy", "rdy",
		      "Ohm", "KOhm", "MOhm",
		      "nF",  "F",
		      "C"};


static void    MM_Start   ();


dual MM_EvalValue (char *Text, PMETEX_VALUE Result)
{
  int  count, Index = 0;
  char Value[9] = "\0";

  PRO_START (" ")

  if (Result == NULL)
    PRO_RETURN ("Invalid Value", FALSE)

  if (! isalpha(Text[0]))
    Text++;

  Result->Unit = TE; 
  while (strncmp(Text, UnitStr[Result->Unit], 2) != 0 && 
	 Result->Unit > BAD_UNIT)
    (Result->Unit)--;

  Result->Value = 0.0;
  Result->Range = BAD_RANGE;
  Result->OL    = TRUE;

  if (Result->Unit > BAD_UNIT)
  {
    Index = 0;
    count = 2;
    switch (Result->Unit)
    {
      case DC :
      case AC :
	strncpy (Value, Text+2, 7);
	Value[7] = '\0';

	if (Text[12] == 'V')
	  if (Text[11] == 'm')
	    Result->Range = MV;
	  else
	    Result->Range = V;
	else
	  if (Text[11] == '')
	    Result->Range = MUA;
	elsif (Text[11] == 'm')
	  Result->Range = MA;
	  else
	    Result->Range = A;
	break;

      case FR : 
	strncpy (Value, Text+2, 7);
	Value[7] = '\0';

	if (Text[10] == 'K')
	  Result->Range = KHZ;
	else
	  Result->Range = MHZ;
	break;

      case LO : 
	if (Text[3] == '-')
	{
	  strcpy (Value, "0.0");
	  Result->Range = LOW;
	}
	else
	{
	  strcpy (Value, "1.0");
	  Result->Range = HIGH;
	}
	break;

      case OH : 
	if (strncmp(Text+5, "O.L", 3) != 0)
	{
	  strncpy (Value, Text+2, 7);
	  Value[7] = '\0';
	}
	if (Text[9] == 'M')
	  Result->Range = MOHM;
	elsif (Text[9] == 'K')
	  Result->Range = KOHM;
	else
	  Result->Range = OHM;
	break;

      case DI : 
	if (strncmp(Text+6, "OL", 2) != 0)
	{
	  strncpy (Value, Text+2, 8);
	  Value[8] = '\0';
	}
	Result->Range = MV;
	break;
      case CA : 
	strncpy (Value, Text+2, 9);
	Value[9] = '\0';
	if (Text[11] == 'n')
	  Result->Range = NF;
	else
	  Result->Range = F;
	break;

      case HF : 
	strncpy (Value, Text+2, 9);
	Value[9] = '\0';
	Result->Range = NO_RANGE;
	break;

      case TE : 
	if (strncmp(Text+6, "OL", 2) != 0)
	{
	  strncpy (Value, Text+2, 9);
	  Value[9] = '\0';
	}
	Result->Range = C;
	break;

      default : 
	PRO0 ("UNKNOWN UNIT")
    }
  }
  if (Value[0] != '\0')
  {
    Result->Value = atof (Value);
    Result->OL = FALSE;
  }

  if (Result->Unit != BAD_UNIT && Result->Range != BAD_RANGE)
    PRO4 ("\\METEX_VALUE: %s %f %s %s\n", 
	  UnitStr[Result->Unit], Result->Value, 
	  RangeStr[Result->Range], Result->OL ? "OL":"OK")
  else
    PRO_RETURN ("PARSE ERROR\n", FALSE)

  PRO_END
  return TRUE;
}


static void MM_CheckData (TIMER_DATA *Data, XtIntervalId Timer)
{
  dual        restart = FALSE;
  char        Text[14];
  METEX_VALUE Value;

  if (MM_HaveData(Data->fd) < 13)
    if (Data->Time < 1000)
      Data->Time += TIMER_STEP;
    elsif (Data->Time < 1500)
      Data->Time += TIMER_STEP;
    else
    {
      (Data->failed)++;
      restart = TRUE;
      PRO0 ("TIMED OUT\n")
    }
  elsif (MM_RecvData(Data->fd, Text, 13) != NULL)
  {
    if (! MM_EvalValue(Text, &Value))
    {
      (Data->failed)++;
      restart = TRUE;
    }
    elsif (Data->Function != NULL)
    {
      PRO1 ("found at %d\n", Data->Time)
      Data->Function (Data->Param, Value);
      return;
    }
  }

  if (((Data->failed+1) % MAX_FAIL) == 0)
    if (DLGOpenTextDialog("recvError", DlgERROR, "Check connection",
			               2, "continue", "abort") == 2)
    {
      MM_ClosePort (Data->fd);
      Data->fd = -1;
    }
    else
    {
      Data->failed = 0;
      MM_Start (Data, -1);
    }
  elsif (restart)
    MM_Start (Data, -1);
  else
    Data->ID = XtAppAddTimeOut (AppCon, TIMER_STEP, 
				(XtTimerCallbackProc)MM_CheckData, Data);
}


static void MM_Start (TIMER_DATA *Data, XtIntervalId Timer)
{
  MM_CleanPort (Data->fd);

  while (! MM_SendData(Data->fd, "D"))
    if (DLGOpenTextDialog("sendError", DlgERROR, "Check connection", 
			               2, "continue", "abort") == 2)
      return;

  Data->Time = 0;
  Data->ID = XtAppAddTimeOut (AppCon, TIMER_STEP, 
			      (XtTimerCallbackProc)MM_CheckData, Data);
}


void MM_ReadValue (TIMER_DATA *Data, TIMERFUNCTION Function, void *Param)
{
  Data->Function = Function;
  Data->Param    = Param;
  Data->failed   = 0;

  if (Data->Delay > 1000)
    Data->ID = 
      XtAppAddTimeOut (AppCon, Data->Delay-1000, 
		               (XtTimerCallbackProc)MM_Start, Data);
  else
    MM_Start (Data, -1);
}


void MM_StopRead (TIMER_DATA *Data)
{
  if (Data->ID != -1)
    XtRemoveTimeOut (Data->ID);
  Data->ID = -1;

  Data->Function = NULL;
  Data->Param    = NULL;
}


