#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <time.h>
#include <termios.h>
#include <unistd.h>

#include <linux/serial.h>

#include "mmGlobal.h"

#define TIME_OUT 1000 /* msec */

typedef struct
{
  int  Baud;
  int  Parity;
  int  Bits;
  int  Stop;
  dual RTS_CTS;
  dual XON_OFF;
} SERIAL_PARAM;

SERIAL_PARAM  Mode = {1200, 0, 7, 2, FALSE, FALSE};
METEX_PARAM   Param = {1, "", 1, FALSE, IMMEDIATE, 0.0, 0};
char          Devices[][16] = {"/dev/ttyS0", "/dev/ttyS1", 
			       "/dev/ttyS2", "/dev/ttyS3"};
static struct termios save;
static char Value[15] = "";
static uid_t r_uid, e_uid;   /* real and effective user ID */
static gid_t r_gid, e_gid;

int MM_OpenPort (char *DevName)
{
  int Result = -1;

  PRO_START ("["__FUNCTION__"]...\n")

  r_uid = getuid ();
  r_gid = getgid ();
  e_uid = geteuid ();
  e_gid = getegid ();

PRO4 ("UID: %d %d  GID: %d %d\n", r_uid, e_uid, r_gid, e_gid);
  setregid (r_gid, e_gid);
  if (setreuid(r_uid, e_uid) < 0)
    PRO2 ("insufficient uid: %d, %d\n", r_uid, e_uid)

/*
   signal (SIGALRM, get_arlm);
   alarm (2);
   init -> savestate
   init
   alarm (0);
   signal (SIGALRM, SIG_IGN);
*/

  Result = open (DevName, O_RDWR);
  if (Result < 0)
    PRO_RETURN ("Error while open serial-device", Result)

  /* save existing attributes */
  tcgetattr (Result, &save);

  PRO_END
  return Result;
}


void MM_ClosePort (int fd)
{
  PRO_START ("["__FUNCTION__"]\n")

  /* restore attributes */ 
  tcsetattr (fd, TCSANOW, &save);

  PRO0 ("closing...")
  if (close(fd) == -1)
    PRO0 ("ERROR: CLOSING failed\n")
  else
    PRO0 ("\\done\n")

  if (setreuid(e_uid, r_uid) < 0)
    PRO2 ("unable to reset uid: %d, %d\n", r_uid, e_uid)

  PRO_END
}


void MM_InitPort (int fd)
{
         int     state;
  struct termios tty;

  PRO_START ("["__FUNCTION__"]...\n")

#if 0
  tcgetattr (fd, &tty);

  PRO0 ("Speed...")
  cfsetospeed (&tty, (speed_t)B1200);
  cfsetispeed (&tty, (speed_t)B1200);  

  PRO0 ("\\Bits...")
  tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS7;

  PRO0 ("\\Flags...")
  tty.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL | IUCLC | IXANY | IXON |
		   IXOFF | INPCK | ISTRIP);
  tty.c_iflag |= (BRKINT | IGNPAR);
  tty.c_oflag &= ~OPOST;
  tty.c_cflag |=  CREAD | CRTSCTS;
  tty.c_lflag  = ~(ICANON | ISIG | ECHO | ECHONL | ECHOE | ECHOK);

  tty.c_cc[VMIN]  = 1;
  tty.c_cc[VTIME] = 5;

  PRO0 ("\\Handshaking...")
  if (! Mode.RTS_CTS)
    tty.c_cflag &= ~CRTSCTS;
  if (Mode.XON_OFF)
    tty.c_iflag |= IXON;

  PRO0 ("\\Parity\n");
  tty.c_cflag &= ~(PARENB | PARODD);
  if (Mode.Stop > 1)
    tty.c_cflag |= CSTOPB;

  tcsetattr (fd, TCSANOW, &tty);

  /* disable RTS */
  if (ioctl (fd, TIOCMGET, &state) < 0)
    PRO0 ("ERROR: TIOCMGET failed !\n")
  state &= ~TIOCM_RTS;
  if (ioctl (fd, TIOCMSET, &state) < 0)
    PRO0 ("ERROR: TIOCMSET failed !\n")
#else
  tcgetattr (fd, &save);
  bzero (&tty, sizeof(tty));

  tty.c_cflag = B1200 | CS7 | CSTOPB | CREAD;
  tty.c_iflag = IGNPAR | BRKINT;
  tty.c_oflag = 0;
  tty.c_lflag = 0;

  tcflush (fd, TCIOFLUSH);

  tty.c_cc[VMIN]  = 1;
  tty.c_cc[VTIME] = 5;

  if (tcsetattr(fd, TCSANOW, &tty) < 0)
  {
    printf ("Error setting serial port\n");
    return;
  }

  /* disable RTS */
  if (ioctl (fd, TIOCMGET, &state) < 0)
    PRO0 ("ERROR: TIOCMGET failed !\n")
  state &= ~TIOCM_RTS;
  if (ioctl (fd, TIOCMSET, &state) < 0)
    PRO0 ("ERROR: TIOCMSET failed !\n")
#endif

  PRO_END
}


void MM_TestPort (int fd)
{
         int     count;
  struct termios tty;

  PRO_START ("["__FUNCTION__"]...\n")

  tcgetattr (fd, &tty);    

  PRO1 ("IFLAGS:  %x\n", tty.c_iflag)
  PRO1 ("OFLAGS:  %x\n", tty.c_oflag)
  PRO1 ("CFLAGS:  %o\n", tty.c_cflag)
  PRO1 ("LFLAGS:  %o\n", tty.c_lflag)
  PRO1 ("LINE:    %x\n", tty.c_line)     /* N_TTY, N_SLIP, N_MOUSE, N_PPP */
  PRO0 ("CONTROL:\n")
  for (count = 0; count < NCCS; ++count)
    PRO1 ("\\%3d ", count)
  PRO0 ("\\\n")
  for (count = 0; count < NCCS; ++count)
    PRO1 ("\\%3d ", tty.c_cc[count])
  PRO0 ("\\\n")

  PRO_END
}


dual MM_SendData (int fd, char *Data)
{
  int len;

  PRO_START ("["__FUNCTION__"]...")

  len = strlen (Data);
  PRO0 ("\\writing...")
  if (write(fd, Data, len) < len)
    PRO_RETURN ("WRITE_ERROR\n", FALSE)
  else
    PRO0 ("\\done...")

  PRO_END
  return TRUE;
}


char *MM_RecvData (int fd, char *Result, int len)
{
  long n = -1;

  PRO_START ("["__FUNCTION__"]...")

  if (ioctl(fd, FIONREAD, &n) == -1)
    PRO_RETURN ("Not enough data", NULL)

  if (read(fd, Result, len) == -1)
    PRO_RETURN ("READ_ERROR", NULL)

  Result[len] = '\0';

  PRO_END
  return Result;
}


int MM_HaveData (int fd)
{
  long i = -1;

  if (ioctl(fd, FIONREAD, &i) == -1)
    return -1;

  return ((int)i);
}


char *MM_GetValue (int fd)
{
  int n;

  PRO_START ("["__FUNCTION__"]...")

  if ((n = MM_HaveData(fd)) >= 14)
  {
    PRO0 ("\\reading...")
    if (read(fd, Value, 14) == -1)
      PRO_RETURN ("READ_ERROR\n", NULL)
    else
      PRO0 ("\\done...")
  }
  else
  {
    PRO1 ("\\#%d...", n)
    PRO_RETURN ("TIMED OUT\n", NULL)
  }

  Value[14] = '\0';

  PRO_END
  return Value;
}



void MM_CleanPort (int fd)
{
  long i = -1;

  PRO_START ("["__FUNCTION__"]...")

  if (ioctl(fd, FIONREAD, &i) == -1)
    PRO_ABORT ("No data found")
  elsif (read(fd, Value, i) == -1)
    PRO_ABORT ("Unable to read data")

  PRO_END
}
	 
