#include <Xm/Xm.h>
#include <X11/Shell.h>

#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>

#include <libdlg.h>

#include "mmGlobal.h"
#include "mmFile.h"

/* TODO:
   mu dynamisch berechnet werden
*/
#define WIN_OFFS 150

#define POSY(y) (WIN_OFFS-(int)(y))

typedef struct
{
  int     PosX, PosY;
  int     SizeX, SizeY;
} GEO_DATA;

typedef struct
{
  DLGELEM *Grid;
  byte     GSize;
} PARAM_DATA;

typedef struct
{
  char        Title[100];

  PLOT_DATA  *Data;
  PARAM_DATA  Param;

  GEO_DATA    Geo;
  TIMER_DATA  Timer;

  Window      Win;
  GC          DrawGC;

  Widget      Shell, Work;
  Widget      ScrWin, ClipWin;
} WIN_DATA,  *PWIN_DATA;


static Arg    args[10];
static double Grid[] = {-1, 0.001, 0.01, 0.1, 1, 10, 100, 1000};
static char  *GridStr[] = {"NONE", 
			   "0.001", "0.01", "0.1", "1", "10", "100", "1000"};


static void MM_HandleEvents ();
/* static void MM_NextValue    (); */
static void MM_StartCB      ();
static void MM_StopCB       ();
static void MM_CloseCB      ();
static void MM_HScrollCB    ();
static void MM_VScrollCB    ();


PWIN_DATA MM_InitPlot (METEX_PARAM Param, int Mode)
{
  PWIN_DATA Result;

  if ((Result = (PWIN_DATA)calloc(sizeof(WIN_DATA),1)) != NULL)
  {
    Result->Title[0]      = '\0';
    Result->Data          = NULL;
    Result->Param.Grid    = NULL;
    Result->Param.GSize   = 1;
    Result->Timer.fd      = -1;
    Result->Timer.Delay   = Param.Delay;
    Result->Timer.ID      = -1;
    Result->Timer.failed  = FALSE;
    Result->DrawGC        = DefaultGC (XDisplay, XScreen);
  }

  return Result;
}


void MM_OpenWindow (char *Title, METEX_PARAM Param, int Mode)
{
  int        Entries;
  Widget     Form, Button;
  Widget     Bottom, Side;
  Widget     Line;
  Widget     HBar, VBar;
  XmString   Label;
  PWIN_DATA  Win = NULL;

  PRO_START ("["__FUNCTION__"]...\n")

  Win = MM_InitPlot (Param, Mode);

  if ((Win->Data = MM_InitData()) == NULL)
    PRO_ABORT ("Initialization failed")

  PRO0 ("Shell...\n")
  XtSetArg (args[0], XmNdeleteResponse, XmDESTROY);
  Win->Shell = XtCreatePopupShell (Title, topLevelShellWidgetClass, 
				          AppShell, args, 1);
  
  XtSetArg (args[0], XmNwidth,  250);
  XtSetArg (args[1], XmNheight, 300);
  Form = XtCreateManagedWidget ("form", xmFormWidgetClass, 
				        Win->Shell, args, 2);

  XtSetArg (args[0], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg (args[1], XmNrightAttachment,  XmATTACH_FORM);
  XtSetArg (args[2], XmNleftAttachment,   XmATTACH_FORM);
  Bottom = XtCreateManagedWidget ("bottom", xmFormWidgetClass, Form, args, 3);

  XtSetArg (args[0], XmNbottomAttachment, XmATTACH_WIDGET);
  XtSetArg (args[1], XmNbottomWidget,     Bottom);
  XtSetArg (args[2], XmNbottomOffset,     3);
  XtSetArg (args[3], XmNleftAttachment,   XmATTACH_FORM);
  XtSetArg (args[4], XmNrightAttachment,  XmATTACH_FORM);
  XtSetArg (args[5], XmNseparatorType,    XmSHADOW_ETCHED_IN); 
  Line = XtCreateManagedWidget ("line", xmSeparatorWidgetClass, Form, args, 6);

  XtSetArg (args[0], XmNrightAttachment,  XmATTACH_FORM);
  XtSetArg (args[1], XmNbottomAttachment, XmATTACH_WIDGET);
  XtSetArg (args[2], XmNbottomWidget,     Bottom);
  XtSetArg (args[3], XmNtopAttachment,    XmATTACH_WIDGET);
  Side = XtCreateManagedWidget ("bottom", xmFormWidgetClass, Form, args, 4);

  XtSetArg (args[0], XmNbottomOffset, 5);

  /* START-BUTTON: */
  Label = XmStringLtoRCreate ("START", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg (args[2], XmNlabelString,    Label);
  Button = XtCreateManagedWidget ("start", xmPushButtonWidgetClass,
				           Bottom, args, 3);
  XtAddCallback (Button, XmNactivateCallback, MM_StartCB, Win);
  XmStringFree (Label);

  /* STOP-BUTTON: */
  Label = XmStringLtoRCreate ("STOP", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (args[1], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg (args[2], XmNleftWidget,     Button);
  XtSetArg (args[3], XmNlabelString,    Label);
  Button = XtCreateManagedWidget ("stop", xmPushButtonWidgetClass,
				          Bottom, args, 4);
  XtAddCallback (Button, XmNactivateCallback, MM_StopCB, Win);
  XmStringFree (Label);

  /* CLOSE-BUTTON: */
  Label = XmStringLtoRCreate ("CLOSE", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (args[1], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg (args[2], XmNlabelString,     Label);
  Button = XtCreateManagedWidget ("close", xmPushButtonWidgetClass,
				           Bottom, args, 3);
  XtAddCallback (Button, XmNactivateCallback, MM_CloseCB, &Win);
  XmStringFree (Label);

#if 0
/*
Elem = DLGMakeScale ("test", 0.0, 100.0, 38.0, 0, TRUE);
Elem = DLGMakeList ("test", 2, 4, "bla1", "bla2", "bla3", "bla4");
Elem = DLGMakeUpDn ("test", 0.0, 100.0, 70.0, 2.0);
Elem = DLGMakeDial ("test", 0.0, 25.0, 0.0, 0, 0);
*/
  Entries = (sizeof(Grid)/sizeof(double));
  Win->Param.Grid = DLGMakeOMenu ("grid", -Entries, GridStr, 4);
  DLGCreateElem (Side, Win->Param.Grid, args, 0);
#endif

  XtManageChild (Win->Shell);
  XtMapWidget (Win->Shell);

  XtSetArg (args[0], XmNscrollingPolicy,        XmAUTOMATIC);
#if 0
  XtSetArg (args[1], XmNscrollBarDisplayPolicy, XmAS_NEEDED);
#else
  XtSetArg (args[1], XmNscrollBarDisplayPolicy, XmSTATIC);
#endif
  XtSetArg (args[2], XmNvisualPolicy,           XmVARIABLE);
  XtSetArg (args[3], XmNtopAttachment,          XmATTACH_FORM);
  XtSetArg (args[4], XmNleftAttachment,         XmATTACH_FORM);
  XtSetArg (args[5], XmNrightAttachment,        XmATTACH_WIDGET);
  XtSetArg (args[6], XmNrightWidget,            Side);
  XtSetArg (args[7], XmNbottomAttachment,       XmATTACH_WIDGET);
  XtSetArg (args[8], XmNbottomWidget,           Line);
  XtSetArg (args[9], XmNbottomOffset,           3);
  Win->ScrWin = XtCreateManagedWidget ("scrWin", xmScrolledWindowWidgetClass,
					         Form, args, 10);

  XtSetArg (args[0], XmNclipWindow,          &Win->ClipWin);
  XtSetArg (args[1], XmNhorizontalScrollBar, &HBar);
  XtSetArg (args[2], XmNverticalScrollBar,   &VBar);
  XtGetValues (Win->ScrWin, args, 3);

  XtAddCallback (HBar, XmNvalueChangedCallback, MM_HScrollCB, &Win);
  XtAddCallback (VBar, XmNvalueChangedCallback, MM_VScrollCB, &Win);

  XtAddEventHandler (Win->ClipWin, StructureNotifyMask, False,
		                   (XtEventHandler)MM_HandleEvents, Win);

  XtSetArg (args[0], XmNtopAttachment,    XmATTACH_FORM);
  XtSetArg (args[1], XmNleftAttachment,   XmATTACH_FORM);
  XtSetArg (args[2], XmNrightAttachment,  XmATTACH_FORM);
  XtSetArg (args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg (args[4], XmNbackground,       BlackPixel(XDisplay, XScreen));
  XtSetArg (args[5], XmNwidth,            400);
  XtSetArg (args[6], XmNheight,           300);
  Win->Work = XtCreateManagedWidget ("area", xmDrawingAreaWidgetClass,
				             Win->ScrWin, args, 7);
  XtAddEventHandler (Win->Work, ExposureMask, False, 
		                (XtEventHandler)MM_HandleEvents, Win);
  Win->Win = XtWindow (Win->Work);

  XtSetArg (args[0], XmNworkWindow, Win->Work);
  XtSetValues (Win->ScrWin, args, 1);

  XtRealizeWidget (Win->Shell);
  XtPopup (Win->Shell, XtGrabNone);

  TOOLMainLoop (AppCon, Win->Shell);

  PRO_END
}


static void MM_DrawGrid (PWIN_DATA Win, int x1, int y1, int x2, int y2)
{
  /*   int    GridNo; */
  double countX;

  XSetForeground (XDisplay, Win->DrawGC, TOOLGetColor("Yellow"));
#if 0
  if ((GridNo = Win->Grid->Value.orec.Result) > 0)
  {
    for (countX = MAX(Min, Grid[GridNo] * Val/Grid[GridNo]);
	 countX < MIN(Max, Grid[GridNo] * Val/Grid[GridNo]); 
	 countX += Grid[GridNo])
      XDrawLine (XDisplay, Win->Win, Win->DrawGC, countX, 0, countX, 1000);
  }
#endif
  for (countX = MAX(x1, x1+100-(x1%100)); countX <= x2; countX += 100.0)
    XDrawLine (XDisplay, Win->Win, Win->DrawGC, countX, y1, countX, y2);
  if ((y1 <= 0) ^ (y2 <= 0))
    XDrawLine (XDisplay, Win->Win, Win->DrawGC, x1, POSY(0), x2, POSY(0));
}


static void MM_Redraw (PWIN_DATA Win, int x1, int y1, int x2, int y2)
{
  int        count, Number;
  int        p1, p2;
  LINE_DATA *Line;

  MM_DrawGrid (Win, x1, y1, x2, y2);

  for (Number = 0; Number < Win->Data->Number; ++Number)
  {
    XSetForeground (XDisplay, Win->DrawGC, TOOLGetColor("Red"));
    count = MAX(0,x1-1);
    Line = Win->Data->Line[Number];
    if (count < Line->len)
      p1 = POSY(Line->Val[count]);
    else
      p1 = POSY (0);
    count++;
    switch (Line->Mode)
    {
      case LINE_PLOT :
	while (count < MIN(x2, Line->len-1))
	{
	  p2 = POSY(Line->Val[count]);
	  if ((p1 >= y1 && p1 <= y2) || (p2 >= y1 && p2 <= y2))
	    XDrawLine (XDisplay, Win->Win, Win->DrawGC,
		                 count, p1, count+1, p2);
	  p1 = p2;
	  count++;
	}
	break;

      case BAR_GRAPH :
	while (count < MIN(x2, Win->Data->Line[Number]->len-1))
	{
	  XDrawLine (XDisplay, Win->Win, Win->DrawGC, 
		     count, POSY(0), count, POSY(Line->Val[count]));
	  count++;
	}
	break;

      default :
        break;
    }
  }
}


static void MM_DrawNew (PWIN_DATA Win, int Number)
{
  int len;

  len = Win->Data->Line[Number]->len;

  XSetForeground (XDisplay, Win->DrawGC, TOOLGetColor("Red"));

  switch (Win->Data->Line[Number]->Mode)
  {
    case LINE_PLOT :
      XDrawLine (XDisplay, Win->Win, Win->DrawGC,
		 MAX(0,len-2), 
		 POSY(Win->Data->Line[Number]->Val[MAX(0,len-2)]),
		 len-1, POSY(Win->Data->Line[Number]->Val[len-1]));
      break;
    case BAR_GRAPH :
      XDrawLine (XDisplay, Win->Win, Win->DrawGC,
		 len-1, POSY(0),
		 len-1, POSY(Win->Data->Line[Number]->Val[len-1]));
      break;

    default :
      break;
  }
}


static void MM_HandleEvents (Widget widget, PWIN_DATA Win, XEvent *event)
{
  switch (event->type)
  {
    case Expose :
      {
	XExposeEvent *expose = (XExposeEvent*)event;

	PRO4 ("EXPOSE: %d - %d / %d - %d\n", 
	      expose->x, expose->x+expose->width,
	      expose->y, expose->y+expose->height)
	MM_Redraw (Win, expose->x, expose->y, 
		        expose->x+expose->width, expose->y+expose->height);
      }
      break;

    case ConfigureNotify :
      {
        Dimension WinX, WinY;

	PRO0 ("CONF NOTIFY: ")
	XtSetArg (args[0], XmNwidth,  &WinX);
	XtSetArg (args[1], XmNheight, &WinY);
	XtGetValues (Win->ClipWin, args, 2);

	PRO2 ("\\%d x %d\n", WinX, WinY)

        if (Win->Data->Number > 0)
	  Win->Geo.SizeX = MAX (WinX, Win->Data->Line[Win->Data->actual]->len);
	else
	  Win->Geo.SizeX = WinX;
	Win->Geo.SizeY = MAX (WinY, 400);

	XtSetArg (args[0], XmNwidth,  Win->Geo.SizeX);
	XtSetArg (args[1], XmNheight, Win->Geo.SizeY);
	XtSetValues (Win->Work, args, 2);
      }
      break;

    default : ;
  }
}


static void MM_Timer (void *Data, METEX_VALUE Value)
{
  PWIN_DATA Win;

  PRO_START ("["__FUNCTION__"]")

  Win = (PWIN_DATA)Data;

  MM_AddValue (Win->Data, Win->Data->actual, Value.Value);

  if (Win->Geo.SizeX < Win->Data->Line[Win->Data->actual]->len)
  {
    /* actualize scrollbar instead of resizing window */
    XtSetArg (args[0], XmNwidth, Win->Data->Line[Win->Data->actual]->len);
    XtSetValues (Win->Work, args, 1);
  }
  MM_DrawNew (Win, Win->Data->actual);

  MM_ReadValue (&Win->Timer, MM_Timer, Win);

  PRO_END
}


static void MM_HScrollCB (Widget widget, PWIN_DATA Win, 
			                 XmScrollBarCallbackStruct *Ptr)
{
  int Min, Max;

  XtSetArg (args[0], XmNminimum,    &Min);
  XtSetArg (args[1], XmNmaximum,    &Max);
  XtSetArg (args[2], XmNsliderSize, &Win->Geo.SizeX);
  XtSetArg (args[3], XmNvalue,      &Win->Geo.PosX);
  XtGetValues (widget, args, 4);
  PRO4 ("H-BAR: %d - %d [%d] = %d\n", Min, Max, Win->Geo.SizeX, Win->Geo.PosX);
}


static void MM_VScrollCB (Widget widget, PWIN_DATA Win, 
			                 XmScrollBarCallbackStruct *Ptr)
{
  int Min, Max;

  XtSetArg (args[0], XmNminimum,    &Min);
  XtSetArg (args[1], XmNmaximum,    &Max);
  XtSetArg (args[2], XmNsliderSize, &Win->Geo.SizeY);
  XtSetArg (args[3], XmNvalue,      &Win->Geo.PosY);
  XtGetValues (widget, args, 4);
  PRO4 ("V-BAR: %d - %d [%d] = %d\n", Min, Max, Win->Geo.SizeY, Win->Geo.PosY);
}


void MM_StartCB (Widget widget, PWIN_DATA Win, XmPushButtonCallbackStruct *Ptr)
{
  int   Number;
  char *DevName = NULL;

  PRO_START ("["__FUNCTION__"]...")

  if (Win->Timer.ID != -1 || Win->Timer.fd != -1)
    PRO_ABORT ("Function allways running")

#if 0
  if (MM_ReadData("test.dat", Win->Data) != NULL)
    MM_Redraw (Win, Win->Geo.PosX, Win->Geo.PosY, 
	            Win->Geo.PosX+Win->Geo.SizeX, 
	            Win->Geo.PosY+Win->Geo.SizeY);
#endif

  if ((Number = MM_AddData(Win->Data, "Plot1", 0, LINE_PLOT)) == -1)
    PRO_ABORT ("Max lines reached")
  Win->Data->actual = Number;

  DevName = (Param.Device == 4)? Param.DevName : Devices[Param.Device];
  if ((Win->Timer.fd = MM_OpenPort(DevName)) == -1)
  {
    PRGPrintf ("opening serial-device failed !");
    PRO_ABORT ("ERROR: failed to open serial-device\n")
  }

  MM_InitPort (Win->Timer.fd);
  MM_TestPort (Win->Timer.fd);

  Win->Timer.Delay = 5000;
  MM_ReadValue (&Win->Timer, MM_Timer, Win);

  PRO_END
}


void MM_StopCB (Widget widget, PWIN_DATA Win, XmPushButtonCallbackStruct *Ptr)
{
  if (Win->Timer.ID != -1)
  {
    XtRemoveTimeOut (Win->Timer.ID);
    Win->Timer.ID = -1;
    if (Win->Timer.fd != -1)
      MM_ClosePort (Win->Timer.fd);
    Win->Timer.fd = -1;
  }

#if 0
  MM_WriteData ("test.dat", Win->Data);
#endif
}


void MM_CloseCB (Widget widget, PWIN_DATA *Win, 
		                XmPushButtonCallbackStruct *Ptr)
{
  if (Win == NULL || *Win == NULL)
    return;

  if ((*Win)->Timer.ID != -1)
  {
    XtRemoveTimeOut ((*Win)->Timer.ID);
    (*Win)->Timer.ID = -1;
  }
  MM_FreeData ((*Win)->Data);

  XtPopdown ((*Win)->Shell);
  XtDestroyWidget ((*Win)->Shell);

  if ((*Win)->Timer.fd != -1)
    MM_ClosePort ((*Win)->Timer.fd);

  if ((*Win)->Param.Grid != NULL)
    DLGFreeElem ((*Win)->Param.Grid);

  free (*Win);
}


void MM_OpenPlotWindow (char *Title, METEX_PARAM Param)
{
  MM_OpenWindow (Title, Param, LINE_PLOT);
}

void MM_OpenBarWindow (char *Title, METEX_PARAM Param)
{
  MM_OpenWindow (Title, Param, BAR_GRAPH);
}

void MM_OpenSegmWindow (char *Title, METEX_PARAM Param)
{
  MM_OpenWindow (Title, Param, SEG_DISPL);
}




